import { ApiCall } from "tsrpc";
import { ReqUpLv, ResUpLv } from "../../shared/protocols/weiwang/PtlUpLv";
import { PlayerFun } from "../../public/player";


export default async function (call: ApiCall<ReqUpLv, ResUpLv>) {
    let data = await G.mongodb.collection("weiwang").findOne({ uid: call.uid });

    if (!data) {
        // 初始化威望数据
        data = (await G.mongodb.collection("weiwang").findOneAndUpdate(
            { uid: call.uid },
            { $set: { hp: 0, atk: 0, def: 0 } },
            { upsert: true, returnDocument: "after" }
        )).value;
    }

    let cur_lv = data[call.req.type];
    let min_lv = Math.min(data.atk, data.def, data.hp);

    let ids = Object.keys(G.gc.renown_level).sort(
        (a, b) => Number(a) - Number(b)
    );

    let config;
    for (let id of ids) {
        config = G.gc.renown_level[id];
        if (min_lv < config.maxlevel) {
            break;
        }
    }

    // 超过档期威望的最大等级
    if (cur_lv + call.req.lv > config.maxlevel) {
        return call.error("", { code: -1, message: lng.weiwang_12 });
    }

    let need = [{ a: config.cost[0].a, t: config.cost[0].t, n: config.cost[0].n * call.req.lv }];

    // 检测消耗是否足够
    await PlayerFun.checkNeedIsMeet(call, need);
    
    // 扣除消耗
    await PlayerFun.cutNeed(call, need);

    data[call.req.type] += call.req.lv;

    // 重新计算buff
    let buff = calc_weiwang_buff(data.atk, data.def, data.hp);
    G.mongodb.collection("weiwang").updateOne({ uid: call.uid }, {
        $set: { buff: buff },
        $inc: { [call.req.type]: call.req.lv },
    });

    call.succ({ lv: { hp: data.hp, atk: data.atk, def: data.def } });
}

/**
 * 计算当前威望buff
*/
function calc_weiwang_buff(atk: number, def: number, hp: number) {
    let buff: k_v<number> = {
        "hp": 0, "def": 0, "atk": 0
    };

    let ids = Object.keys(G.gc.renown_level).sort(
        (a, b) => Number(a) - Number(b)
    );

    for (let i = 0; i < ids.length; i++) {
        let curr = G.gc.renown_level[ids[i]];
        if (i == 0) {
            buff["hp"] += curr.hp * Math.min(curr.maxlevel, hp);
            buff["atk"] += curr.atk * Math.min(curr.maxlevel, atk);
            buff["def"] += curr.def * Math.min(curr.maxlevel, def);
        } else {
            let prev = G.gc.renown_level[ids[i - 1]];

            if (hp > prev.maxlevel) {
                buff["hp"] += curr.hp * Math.min(hp - prev.maxlevel, curr.maxlevel - prev.maxlevel);
            }

            if (atk > prev.maxlevel) {
                buff["atk"] += curr.atk * Math.min(atk - prev.maxlevel, curr.maxlevel - prev.maxlevel);
            }

            if (def > prev.maxlevel) {
                buff["def"] += curr.def * Math.min(def - prev.maxlevel, curr.maxlevel - prev.maxlevel);
            }
        }
    }

    return buff;
}